@@ -58,7 +58,8 @@ BookInfo::BookInfo(Metadata* parent)
     : QObject(parent)
     , d(new Private)
 {
-    qRegisterMetaType<BookInfo*>("BookInfo*");
+    static const int typeId = qRegisterMetaType<BookInfo*>("BookInfo*");
+    Q_UNUSED(typeId);
     d->coverPage = new Page(metadata()->document());
     d->coverPage->setIsCoverPage(true);
 }
@@ -74,7 +75,7 @@ void BookInfo::toXml(QXmlStreamWriter* writer)
 {
     writer->writeStartElement(QStringLiteral("book-info"));
 
-    Q_FOREACH(Author* author, d->author) {
+    for(Author* author : d->author) {
         author->toXml(writer);
     }
 
@@ -98,7 +99,7 @@ void BookInfo::toXml(QXmlStreamWriter* writer)
 
     writer->writeStartElement("characters");
     writer->writeCharacters("");
-    Q_FOREACH(const QString& character, d->characters) {
+    for(const QString& character : d->characters) {
         writer->writeStartElement("name");
         writer->writeCharacters(character);
         writer->writeEndElement();
@@ -110,9 +111,10 @@ void BookInfo::toXml(QXmlStreamWriter* writer)
         annotations.next();
         writer->writeStartElement(QStringLiteral("annotation"));
         writer->writeAttribute(QStringLiteral("lang"), annotations.key());
-        Q_FOREACH(const QString& paragraph, annotations.value()) {
+        for(const QString& paragraph : annotations.value()) {
             writer->writeStartElement(QStringLiteral("p"));
-            writer->writeCharacters(paragraph);
+            writer->writeCharacters("");  // to ensure we close the tag correctly and don't end up with a <p />
+            writer->device()->write(paragraph.toUtf8().constData(), paragraph.toUtf8().length());
             writer->writeEndElement();
         }
         writer->writeEndElement();
@@ -130,18 +132,18 @@ void BookInfo::toXml(QXmlStreamWriter* writer)
     d->coverPage->toXml(writer);
 
     writer->writeStartElement(QStringLiteral("languages"));
-    Q_FOREACH(Language* language, d->languages) {
+    for(Language* language : d->languages) {
         language->toXml(writer);
     }
     writer->writeEndElement();
 
-    Q_FOREACH(Sequence* sequence, d->sequence) {
+    for(Sequence* sequence : d->sequence) {
         sequence->toXml(writer);
     }
-    Q_FOREACH(DatabaseRef* ref, d->databaseRef) {
+    for(DatabaseRef* ref : d->databaseRef) {
         ref->toXml(writer);
     }
-    Q_FOREACH(ContentRating* rating, d->contentRating) {
+    for(ContentRating* rating : d->contentRating) {
         rating->toXml(writer);
     }
     
@@ -156,7 +158,7 @@ void BookInfo::toXml(QXmlStreamWriter* writer)
     writer->writeEndElement();
 }
 
-bool BookInfo::fromXml(QXmlStreamReader *xmlReader)
+bool BookInfo::fromXml(QXmlStreamReader *xmlReader, const QString& xmlData)
 {
     while(xmlReader->readNextStartElement())
     {
@@ -196,7 +198,15 @@ bool BookInfo::fromXml(QXmlStreamReader *xmlReader)
             QStringList paragraphs;
             while(xmlReader->readNextStartElement()) {
                 if(xmlReader->name() == QStringLiteral("p")) {
-                    paragraphs.append(xmlReader->readElementText(QXmlStreamReader::IncludeChildElements));
+                    int startPoint = xmlReader->characterOffset();
+                    int endPoint{startPoint};
+                    while(xmlReader->readNext()) {
+                        if (xmlReader->isEndElement() && xmlReader->name() == QStringLiteral("p")) {
+                            endPoint = xmlReader->characterOffset();
+                            break;
+                        }
+                    }
+                    paragraphs.append(xmlData.mid(startPoint, endPoint - startPoint - 4));
                 }
                 else {
                     xmlReader->skipCurrentElement();
@@ -211,7 +221,7 @@ bool BookInfo::fromXml(QXmlStreamReader *xmlReader)
         }
         else if(xmlReader->name() == QStringLiteral("coverpage"))
         {
-            if(!d->coverPage->fromXml(xmlReader)) {
+            if(!d->coverPage->fromXml(xmlReader, xmlData)) {
                 return false;
             }
         }
@@ -548,15 +558,19 @@ void BookInfo::setCoverpage(Page* newCover)
     d->coverPage = newCover;
 }
 
-QList<Language *> BookInfo::languages()
+QObjectList BookInfo::languages()
 {
-    return d->languages;
+    QObjectList list;
+    for (Language* language : d->languages) {
+        list << language;
+    }
+    return list;
 }
 
 void BookInfo::addLanguage(Language* language)
 {
     d->languages.append(language);
-    emit languageEntryListChanged();
+    Q_EMIT languagesChanged();
 }
 
 QStringList BookInfo::languageEntryList()
@@ -579,7 +593,7 @@ void BookInfo::addLanguage(QString language, bool show)
 void BookInfo::removeLanguage(Language* language)
 {
     d->languages.removeAll(language);
-    emit languageEntryListChanged();
+    Q_EMIT languagesChanged();
 }
 
 void BookInfo::removeLanguage(int index)
