@@ -26,24 +26,29 @@
 #include <AcbfBody.h>
 #include <AcbfBookinfo.h>
 #include <AcbfDocument.h>
+#include <AcbfDocumentinfo.h>
 #include <AcbfMetadata.h>
 #include <AcbfPage.h>
 #include <AcbfPublishinfo.h>
-#include <AcbfDocumentinfo.h>
+#include <AcbfStyleSheet.h>
 
 #include <QCoreApplication>
 #include <QDir>
+#include <QFontDatabase>
+#include <QImageReader>
 #include <QMimeDatabase>
 #include <QQmlEngine>
 #include <QTemporaryFile>
 #include <QXmlStreamReader>
 
 #include <KFileMetaData/UserMetaData>
+#include <KLocalizedString>
 #include <karchive.h>
 #include <kzip.h>
 #include "KRar.h" // "" because it's a custom thing for now
 
 #include <qtquick_debug.h>
+#include <AcbfData.h>
 
 class ArchiveBookModel::Private
 {
@@ -57,13 +62,48 @@ public:
         , isDirty(false)
         , isLoading(false)
     {}
+    ~Private() {
+        for (int fontId : fontIdByFilename.values()) {
+            fontDatabase.removeApplicationFont(fontId);
+        }
+        delete archive;
+    }
     ArchiveBookModel* q;
     QQmlEngine* engine;
     KArchive* archive;
+    QStringList fileEntries;
+    QStringList fileEntriesToDelete;
+    QHash<QString, const KArchiveFile*> archiveFiles;
     bool readWrite;
     ArchiveImageProvider* imageProvider;
     bool isDirty;
     bool isLoading;
+    QMimeDatabase mimeDatabase;
+    QFontDatabase fontDatabase;
+    QHash<QString, int> fontIdByFilename;
+    QString acbfEntryName;
+
+    void closeBook() {
+        q->beginResetModel();
+        if(archive)
+        {
+            q->clearPages();
+            archiveFiles.clear();
+            archive->close();
+            delete archive;
+            archive = nullptr;
+        }
+        if(imageProvider && engine) {
+            engine->removeImageProvider(imageProvider->prefix());
+        }
+        imageProvider = nullptr;
+        fileEntries.clear();
+        Q_EMIT q->fileEntriesChanged();
+        fileEntriesToDelete.clear();
+        Q_EMIT q->fileEntriesToDeleteChanged();
+        q->endResetModel();
+        acbfEntryName.clear();
+    }
 
     static int counter()
     {
@@ -134,6 +174,7 @@ ArchiveBookModel::ArchiveBookModel(QObject* parent)
 
 ArchiveBookModel::~ArchiveBookModel()
 {
+    d->closeBook();
     delete d;
 }
 
@@ -141,14 +182,14 @@ QStringList recursiveEntries(const KArchiveDirectory* dir)
 {
     QStringList entries = dir->entries();
     QStringList allEntries = entries;
-    Q_FOREACH(const QString& entryName, entries)
+    for(const QString& entryName : entries)
     {
         const KArchiveEntry* entry = dir->entry(entryName);
         if(entry->isDirectory())
         {
             const KArchiveDirectory* subDir = static_cast<const KArchiveDirectory*>(entry);
             QStringList subEntries = recursiveEntries(subDir);
-            Q_FOREACH(const QString& subEntry, subEntries)
+            for(const QString& subEntry : subEntries)
             {
                 entries.append(entryName + "/" + subEntry);
             }
@@ -161,20 +202,10 @@ void ArchiveBookModel::setFilename(QString newFilename)
 {
     setProcessing(true);
     d->isLoading = true;
+    d->closeBook();
+    beginResetModel();
 
-    if(d->archive)
-    {
-        clearPages();
-        delete d->archive;
-    }
-    d->archive = nullptr;
-    if(d->imageProvider && d->engine) {
-        d->engine->removeImageProvider(d->imageProvider->prefix());
-    }
-    d->imageProvider = nullptr;
-
-    QMimeDatabase db;
-    QMimeType mime = db.mimeTypeForFile(newFilename);
+    QMimeType mime = d->mimeDatabase.mimeTypeForFile(newFilename);
     if(mime.inherits("application/zip"))
     {
         d->archive = new KZip(newFilename);
@@ -190,6 +221,7 @@ void ArchiveBookModel::setFilename(QString newFilename)
         QString prefix = QString("archivebookpage%1").arg(QString::number(Private::counter()));
         if(d->archive->open(QIODevice::ReadOnly))
         {
+            QMutexLocker locker(&archiveMutex);
             d->imageProvider = new ArchiveImageProvider();
             d->imageProvider->setArchiveBookModel(this);
             d->imageProvider->setPrefix(prefix);
@@ -197,21 +229,22 @@ void ArchiveBookModel::setFilename(QString newFilename)
                 d->engine->addImageProvider(prefix, d->imageProvider);
             }
 
-            QStringList entries = recursiveEntries(d->archive->directory());
+            d->fileEntries = recursiveEntries(d->archive->directory());
+            d->fileEntries.sort();
+            Q_EMIT fileEntriesChanged();
 
             // First check and see if we've got an ACBF document in there...
-            QString acbfEntry;
             QString comicInfoEntry;
             QStringList xmlFiles;
             QLatin1String acbfSuffix(".acbf");
             QLatin1String ComicInfoXML("comicinfo.xml");
             QLatin1String xmlSuffix(".xml");
             QStringList images;
-            Q_FOREACH(const QString& entry, entries)
+            for(const QString& entry : d->fileEntries)
             {
                 if(entry.toLower().endsWith(acbfSuffix))
                 {
-                    acbfEntry = entry;
+                    d->acbfEntryName = entry;
                     break;
                 }
                 if(entry.toLower().endsWith(xmlSuffix)) {
@@ -222,20 +255,20 @@ void ArchiveBookModel::setFilename(QString newFilename)
                     }
                 }
                 if (entry.toLower().endsWith(".jpg") || entry.toLower().endsWith(".jpeg")
-                        || entry.toLower().endsWith(".gif") || entry.toLower().endsWith(".png")) {
+                        || entry.toLower().endsWith(".gif") || entry.toLower().endsWith(".png") || entry.toLower().endsWith(".webp")) {
                     images.append(entry);
                 }
             }
             images.sort();
-            if(!acbfEntry.isEmpty())
+            if(!d->acbfEntryName.isEmpty())
             {
                 AdvancedComicBookFormat::Document* acbfDocument = new AdvancedComicBookFormat::Document(this);
-                const KArchiveFile* archFile = d->archive->directory()->file(acbfEntry);
+                const KArchiveFile* archFile = d->archive->directory()->file(d->acbfEntryName);
                 if(acbfDocument->fromXml(QString(archFile->data())))
                 {
                     setAcbfData(acbfDocument);
                     addPage(QString("image://%1/%2").arg(prefix).arg(acbfDocument->metaData()->bookInfo()->coverpage()->imageHref()), acbfDocument->metaData()->bookInfo()->coverpage()->title());
-                    Q_FOREACH(AdvancedComicBookFormat::Page* page, acbfDocument->body()->pages())
+                    for(AdvancedComicBookFormat::Page* page : acbfDocument->body()->pages())
                     {
                         addPage(QString("image://%1/%2").arg(prefix).arg(page->imageHref()), page->title());
                     }
@@ -261,7 +294,7 @@ void ArchiveBookModel::setFilename(QString newFilename)
                     setAcbfData(acbfDocument);
                     QString undesired = QString("%1").arg("/").append("Thumbs.db");
                     addPage(QString("image://%1/%2").arg(prefix).arg(acbfDocument->metaData()->bookInfo()->coverpage()->imageHref()), acbfDocument->metaData()->bookInfo()->coverpage()->title());
-                    Q_FOREACH(AdvancedComicBookFormat::Page* page, acbfDocument->body()->pages())
+                    for(AdvancedComicBookFormat::Page* page : acbfDocument->body()->pages())
                     {
                         addPage(QString("image://%1/%2").arg(prefix).arg(page->imageHref()), page->title());
                     }
@@ -270,9 +303,8 @@ void ArchiveBookModel::setFilename(QString newFilename)
             if(!acbfData())
             {
                 // fall back to just handling the files directly if there's no ACBF document...
-                entries.sort();
                 QString undesired = QString("%1").arg("/").append("Thumbs.db");
-                Q_FOREACH(const QString& entry, entries)
+                for(const QString& entry : qAsConst(d->fileEntries))
                 {
                     const KArchiveEntry* archEntry = d->archive->directory()->entry(entry);
                     if(archEntry->isFile() && !entry.endsWith(undesired))
@@ -293,7 +325,7 @@ void ArchiveBookModel::setFilename(QString newFilename)
 //     if(dir.exists())
 //     {
 //         QFileInfoList entries = dir.entryInfoList(QDir::Files, QDir::Name);
-//         Q_FOREACH(const QFileInfo& entry, entries)
+//         for(const QFileInfo& entry : entries)
 //         {
 //             addPage(QString("file://").append(entry.canonicalFilePath()), entry.fileName());
 //         }
@@ -312,6 +344,7 @@ void ArchiveBookModel::setFilename(QString newFilename)
     d->isLoading = false;
     emit loadingCompleted(success);
     setProcessing(false);
+    endResetModel();
 }
 
 QString ArchiveBookModel::author() const
@@ -437,11 +470,78 @@ void ArchiveBookModel::setDirty(bool isDirty)
     emit hasUnsavedChangesChanged();
 }
 
+QStringList ArchiveBookModel::fileEntries() const
+{
+    return d->fileEntries;
+}
+
+int ArchiveBookModel::fileEntryReferenced(const QString& fileEntry) const
+{
+    int isReferenced{0};
+    AdvancedComicBookFormat::Document* document = qobject_cast<AdvancedComicBookFormat::Document*>(acbfData());
+    if (document->metaData()->bookInfo()->coverpage()->imageHref() == fileEntry) {
+        isReferenced = 1;
+    }
+    if (!isReferenced) {
+        for(const AdvancedComicBookFormat::Page* page : document->body()->pages()) {
+            if (page->imageHref() == fileEntry) {
+                isReferenced = 1;
+                break;
+            }
+        }
+    }
+    if (!isReferenced) {
+        for(const QObject* obj : document->styleSheet()->styles()) {
+            const AdvancedComicBookFormat::Style* style = qobject_cast<const AdvancedComicBookFormat::Style*>(obj);
+            const QString styleString{style->toString()};
+            if (styleString.contains(fileEntry)) {
+                isReferenced = 1;
+                break;
+            } else if (styleString.contains(fileEntry.split("/").last())) {
+                isReferenced = 2;
+                break;
+            }
+        }
+    }
+    return isReferenced;
+}
+
+bool ArchiveBookModel::fileEntryIsDirectory(const QString& fileEntry) const
+{
+    bool isDirectory{false};
+    const KArchiveEntry* entry = d->archive->directory()->entry(fileEntry);
+    if (entry && entry->isDirectory()) {
+        isDirectory = true;
+    }
+    return isDirectory;
+}
+
+QStringList ArchiveBookModel::fileEntriesToDelete() const
+{
+    return d->fileEntriesToDelete;
+}
+
+void ArchiveBookModel::markArchiveFileForDeletion(const QString& archiveFile, bool markForDeletion)
+{
+    if(markForDeletion) {
+        if (!d->fileEntriesToDelete.contains(archiveFile)) {
+            d->fileEntriesToDelete << archiveFile;
+            Q_EMIT fileEntriesToDeleteChanged();
+        }
+    } else {
+        if (d->fileEntriesToDelete.contains(archiveFile)) {
+            d->fileEntriesToDelete.removeAll(archiveFile);
+            Q_EMIT fileEntriesToDeleteChanged();
+        }
+    }
+}
+
 bool ArchiveBookModel::saveBook()
 {
     bool success = true;
     if(d->isDirty)
     {
+        QMutexLocker locker(&archiveMutex);
         // TODO get new filenames out of acbf
 
         setProcessing(true);
@@ -452,43 +552,49 @@ bool ArchiveBookModel::saveBook()
         QString archiveFileName = tmpFile.fileName().append(".cbz");
         QFileInfo fileInfo(tmpFile);
         tmpFile.close();
-        qCDebug(QTQUICK_LOG) << "Creating archive in" << archiveFileName;
+        setProcessingDescription(i18n("Creating archive in %1", archiveFileName));
         KZip* archive = new KZip(archiveFileName);
         archive->open(QIODevice::ReadWrite);
 
+        QString acbfFileName{d->acbfEntryName};
+        if (acbfFileName.isEmpty()) {
+            acbfFileName = QStringLiteral("metadata.acbf");
+        } else {
+            // If we actually /have/ an acbf filename already, let's not copy the old one across...
+            d->fileEntriesToDelete << acbfFileName;
+        }
         // We're a zip file... size isn't used
-        qCDebug(QTQUICK_LOG) << "Writing in ACBF data";
-        archive->prepareWriting("metadata.acbf", fileInfo.owner(), fileInfo.group(), 0);
+        setProcessingDescription(i18n("Writing in ACBF data"));
+        archive->prepareWriting(acbfFileName, fileInfo.owner(), fileInfo.group(), 0);
         AdvancedComicBookFormat::Document* acbfDocument = qobject_cast<AdvancedComicBookFormat::Document*>(acbfData());
         if(!acbfDocument)
         {
             acbfDocument = d->createNewAcbfDocumentFromLegacyInformation();
         }
-        QString acbfString = acbfDocument->toXml();
-        archive->writeData(acbfString.toUtf8(), acbfString.size());
-        archive->finishWriting(acbfString.size());
+        QByteArray acbfStringUtf8 = acbfDocument->toXml().toUtf8();
+        archive->writeData(acbfStringUtf8, acbfStringUtf8.size());
+        archive->finishWriting(acbfStringUtf8.size());
 
-        qCDebug(QTQUICK_LOG) << "Copying across cover page";
-        const KArchiveFile* archFile = archiveFile(acbfDocument->metaData()->bookInfo()->coverpage()->imageHref());
-        if(archFile)
-        {
-            archive->prepareWriting(acbfDocument->metaData()->bookInfo()->coverpage()->imageHref(), fileInfo.owner(), fileInfo.group(), 0);
-            archive->writeData(archFile->data(), archFile->size());
-            archive->finishWriting(archFile->size());
-        }
-
-        Q_FOREACH(AdvancedComicBookFormat::Page* page, acbfDocument->body()->pages())
-        {
+        setProcessingDescription(i18n("Copying across all files not marked for deletion"));
+        const QStringList allFiles = fileEntries();
+        const KArchiveFile* archFile{nullptr};
+        for( const QString& file : allFiles) {
             qApp->processEvents();
-            qCDebug(QTQUICK_LOG) << "Copying over" << page->title();
-            archFile = archiveFile(page->imageHref());
-            if(archFile)
-            {
-                archive->prepareWriting(page->imageHref(), archFile->user(), archFile->group(), 0);
-                archive->writeData(archFile->data(), archFile->size());
-                archive->finishWriting(archFile->size());
+            if (d->fileEntriesToDelete.contains(file)) {
+                qCDebug(QTQUICK_LOG) << "Not copying file marked for deletion:" << file;
+            } else {
+                setProcessingDescription(i18n("Copying over %1", file));
+                archFile = archiveFile(file);
+                if(archFile && archFile->isFile())
+                {
+                    archive->prepareWriting(file, archFile->user(), archFile->group(), 0);
+                    archive->writeData(archFile->data(), archFile->size());
+                    archive->finishWriting(archFile->size());
+                }
             }
         }
+        d->fileEntriesToDelete.clear();
+        Q_EMIT fileEntriesToDeleteChanged();
 
         archive->close();
         qCDebug(QTQUICK_LOG) << "Archive created and closed...";
@@ -497,7 +603,7 @@ bool ArchiveBookModel::saveBook()
         beginResetModel();
 
         QString actualFile = d->archive->fileName();
-        d->archive->close();
+        d->closeBook();
 
         // This seems roundabout... but it retains ctime and xattrs, which would be gone
         // if we just did a delete+rename
@@ -506,7 +612,7 @@ bool ArchiveBookModel::saveBook()
         {
             QFile originFile(archiveFileName);
             if(originFile.open(QIODevice::ReadOnly)) {
-                qCDebug(QTQUICK_LOG) << "Copying all content from" << archiveFileName << "to" << actualFile;
+                setProcessingDescription(i18n("Copying all content from %1 to %2", archiveFileName, actualFile));
                 while(!originFile.atEnd())
                 {
                     destinationFile.write(originFile.read(65536));
@@ -516,9 +622,11 @@ bool ArchiveBookModel::saveBook()
                 originFile.close();
                 if(originFile.remove())
                 {
-                    qCDebug(QTQUICK_LOG) << "Success! Now loading the new archive...";
+                    setProcessingDescription(i18n("Successfully replaced old archive with the new archive - now loading the new archive..."));
                     // now load the new thing...
+                    locker.unlock();
                     setFilename(actualFile);
+                    locker.relock();
                 }
                 else
                 {
@@ -581,14 +689,24 @@ void ArchiveBookModel::removePage(int pageNumber)
         {
             if(pageNumber == 0)
             {
-                //Page no 0 is the cover page, when removed we'll take the next page.
+                // Page no 0 is the cover page, when removed we'd usually end up with no cover page
+                // Normally we'd want to discourage this, but we do need to support the functionality
+                AdvancedComicBookFormat::Page* cover = acbfDocument->metaData()->bookInfo()->coverpage();
+                if (cover) {
+                    cover->deleteLater();
+                }
                 AdvancedComicBookFormat::Page* page = acbfDocument->body()->page(0);
                 acbfDocument->metaData()->bookInfo()->setCoverpage(page);
-                acbfDocument->body()->removePage(page);
+                if (page) {
+                    acbfDocument->body()->removePage(page);
+                }
             }
             else {
                 AdvancedComicBookFormat::Page* page = acbfDocument->body()->page(pageNumber-1);
-                acbfDocument->body()->removePage(page);
+                if (page) {
+                    acbfDocument->body()->removePage(page);
+                    page->deleteLater();
+                }
             }
         }
     }
@@ -615,6 +733,9 @@ void ArchiveBookModel::addPageFromFile(QString fileUrl, int insertAfter)
         d->archive->close();
         d->archive->open(QIODevice::ReadOnly);
         addPage(QString("image://%1/%2").arg(d->imageProvider->prefix()).arg(archiveFileName), archiveFileName.split("/").last());
+        d->fileEntries << archiveFileName;
+        d->fileEntries.sort();
+        Q_EMIT fileEntriesChanged();
         saveBook();
     }
 }
@@ -684,6 +805,9 @@ QString ArchiveBookModel::createBook(QString folder, QString title, QString cove
     model->d->archive->close();
     model->d->archive->open(QIODevice::ReadWrite);
     model->d->archive->addLocalFile(coverUrl, coverArchiveName);
+    model->d->fileEntries << coverArchiveName;
+    model->d->fileEntries.sort();
+    Q_EMIT model->fileEntriesChanged();
     model->d->archive->close();
 
     model->deleteLater();
@@ -693,11 +817,14 @@ QString ArchiveBookModel::createBook(QString folder, QString title, QString cove
     return filename;
 }
 
-const KArchiveFile * ArchiveBookModel::archiveFile(const QString& filePath)
+const KArchiveFile * ArchiveBookModel::archiveFile(const QString& filePath) const
 {
     if(d->archive)
     {
-        return d->archive->directory()->file(filePath);
+        if(!d->archiveFiles.contains(filePath)) {
+            d->archiveFiles[filePath] = d->archive->directory()->file(filePath);
+        }
+        return d->archiveFiles[filePath];
     }
     return nullptr;
 }
@@ -881,70 +1008,70 @@ bool ArchiveBookModel::loadComicInfoXML(QString xmlDocument, QObject *acbfData,
                 //Authors...
                 else if(xmlReader.name() == QStringLiteral("Writer"))
                 {
-                    QStringList people = xmlReader.readElementText().split(",", QString::SkipEmptyParts);
+                    QStringList people = xmlReader.readElementText().split(",", Qt::SkipEmptyParts);
                     for (int i=0; i < people.size(); i++) {
                         acbfDocument->metaData()->bookInfo()->addAuthor("Writer", "", "", "", "", people.at(i).trimmed(), empty, empty);
                     }
                 }
                 else if(xmlReader.name() == QStringLiteral("Plotter"))
                 {
-                    QStringList people = xmlReader.readElementText().split(",", QString::SkipEmptyParts);
+                    QStringList people = xmlReader.readElementText().split(",", Qt::SkipEmptyParts);
                     for (int i=0; i < people.size(); i++) {
                         acbfDocument->metaData()->bookInfo()->addAuthor("Writer", "", "", "", "", people.at(i).trimmed(), empty, empty);
                     }
                 }
                 else if(xmlReader.name() == QStringLiteral("Scripter"))
                 {
-                    QStringList people = xmlReader.readElementText().split(",", QString::SkipEmptyParts);
+                    QStringList people = xmlReader.readElementText().split(",", Qt::SkipEmptyParts);
                     for (int i=0; i < people.size(); i++) {
                         acbfDocument->metaData()->bookInfo()->addAuthor("Writer", "", "", "", "", people.at(i).trimmed(), empty, empty);
                     }
                 }
                 else if(xmlReader.name() == QStringLiteral("Penciller"))
                 {
-                    QStringList people = xmlReader.readElementText().split(",", QString::SkipEmptyParts);
+                    QStringList people = xmlReader.readElementText().split(",", Qt::SkipEmptyParts);
                     for (int i=0; i < people.size(); i++) {
                         acbfDocument->metaData()->bookInfo()->addAuthor("Penciller", "", "", "", "", people.at(i).trimmed(), empty, empty);
                     }
                 }
                 else if(xmlReader.name() == QStringLiteral("Inker"))
                 {
-                    QStringList people = xmlReader.readElementText().split(",", QString::SkipEmptyParts);
+                    QStringList people = xmlReader.readElementText().split(",", Qt::SkipEmptyParts);
                     for (int i=0; i < people.size(); i++) {
                         acbfDocument->metaData()->bookInfo()->addAuthor("Inker", "", "", "", "", people.at(i).trimmed(), empty, empty);
                     }
                 }
                 else if(xmlReader.name() == QStringLiteral("Colorist"))
                 {
-                    QStringList people = xmlReader.readElementText().split(",", QString::SkipEmptyParts);
+                    QStringList people = xmlReader.readElementText().split(",", Qt::SkipEmptyParts);
                     for (int i=0; i < people.size(); i++) {
                         acbfDocument->metaData()->bookInfo()->addAuthor("Colorist", "", "", "", "", people.at(i).trimmed(), empty, empty);
                     }
                 }
                 else if(xmlReader.name() == QStringLiteral("CoverArtist"))
                 {
-                    QStringList people = xmlReader.readElementText().split(",", QString::SkipEmptyParts);
+                    QStringList people = xmlReader.readElementText().split(",", Qt::SkipEmptyParts);
                     for (int i=0; i < people.size(); i++) {
                         acbfDocument->metaData()->bookInfo()->addAuthor("CoverArtist", "", "", "", "", people.at(i).trimmed(), empty, empty);
                     }
                 }
                 else if(xmlReader.name() == QStringLiteral("Letterer"))
                 {
-                    QStringList people = xmlReader.readElementText().split(",", QString::SkipEmptyParts);
+                    QStringList people = xmlReader.readElementText().split(",", Qt::SkipEmptyParts);
                     for (int i=0; i < people.size(); i++) {
                         acbfDocument->metaData()->bookInfo()->addAuthor("Letterer", "", "", "", "", people.at(i).trimmed(), empty, empty);
                     }
                 }
                 else if(xmlReader.name() == QStringLiteral("Editor"))
                 {
-                    QStringList people = xmlReader.readElementText().split(",", QString::SkipEmptyParts);
+                    QStringList people = xmlReader.readElementText().split(",", Qt::SkipEmptyParts);
                     for (int i=0; i < people.size(); i++) {
                         acbfDocument->metaData()->bookInfo()->addAuthor("Editor", "", "", "", "", people.at(i).trimmed(), empty, empty);
                     }
                 }
                 else if(xmlReader.name() == QStringLiteral("Other"))
                 {
-                    QStringList people = xmlReader.readElementText().split(",", QString::SkipEmptyParts);
+                    QStringList people = xmlReader.readElementText().split(",", Qt::SkipEmptyParts);
                     for (int i=0; i < people.size(); i++) {
                         acbfDocument->metaData()->bookInfo()->addAuthor("Other", "", "", "", "", people.at(i).trimmed(), empty, empty);
                     }
@@ -952,7 +1079,7 @@ bool ArchiveBookModel::loadComicInfoXML(QString xmlDocument, QObject *acbfData,
                 //Characters...
                 else if(xmlReader.name() == QStringLiteral("Characters"))
                 {
-                    QStringList people = xmlReader.readElementText().split(",", QString::SkipEmptyParts);
+                    QStringList people = xmlReader.readElementText().split(",", Qt::SkipEmptyParts);
                     for (int i=0; i < people.size(); i++) {
                         acbfDocument->metaData()->bookInfo()->addCharacter(people.at(i).trimmed());
                     }
@@ -960,28 +1087,28 @@ bool ArchiveBookModel::loadComicInfoXML(QString xmlDocument, QObject *acbfData,
                 //Throw the rest into the keywords.
                 else if(xmlReader.name() == QStringLiteral("Teams"))
                 {
-                    QStringList teams = xmlReader.readElementText().split(",", QString::SkipEmptyParts);
+                    QStringList teams = xmlReader.readElementText().split(",", Qt::SkipEmptyParts);
                     for (int i=0; i < teams.size(); i++) {
                         keywords.append(teams.at(i).trimmed());
                     }
                 }
                 else if(xmlReader.name() == QStringLiteral("Locations"))
                 {
-                    QStringList locations = xmlReader.readElementText().split(",", QString::SkipEmptyParts);
+                    QStringList locations = xmlReader.readElementText().split(",", Qt::SkipEmptyParts);
                     for (int i=0; i < locations.size(); i++) {
                         keywords.append(locations.at(i).trimmed());
                     }
                 }
                 else if(xmlReader.name() == QStringLiteral("StoryArc"))
                 {
-                    QStringList arc = xmlReader.readElementText().split(",", QString::SkipEmptyParts);
+                    QStringList arc = xmlReader.readElementText().split(",", Qt::SkipEmptyParts);
                     for (int i=0; i < arc.size(); i++) {
                         keywords.append(arc.at(i).trimmed());
                     }
                 }
                 else if(xmlReader.name() == QStringLiteral("SeriesGroup"))
                 {
-                    QStringList group = xmlReader.readElementText().split(",", QString::SkipEmptyParts);
+                    QStringList group = xmlReader.readElementText().split(",", Qt::SkipEmptyParts);
                     for (int i=0; i < group.size(); i++) {
                         keywords.append(group.at(i).trimmed());
                     }
@@ -1059,7 +1186,8 @@ bool ArchiveBookModel::loadCoMet(QStringList xmlDocuments, QObject *acbfData, QS
 {
     KFileMetaData::UserMetaData filedata(filename);
     AdvancedComicBookFormat::Document* acbfDocument = qobject_cast<AdvancedComicBookFormat::Document*>(acbfData);
-    Q_FOREACH(const QString xmlDocument, xmlDocuments) {
+    for(const QString& xmlDocument : qAsConst(xmlDocuments)) {
+        QMutexLocker locker(&archiveMutex);
         const KArchiveFile* archFile = d->archive->directory()->file(xmlDocument);
         QXmlStreamReader xmlReader(archFile->data());
         if(xmlReader.readNextStartElement())
@@ -1244,7 +1372,7 @@ bool ArchiveBookModel::loadCoMet(QStringList xmlDocuments, QObject *acbfData, QS
                         cover->setImageHref(url);
                         acbfDocument->metaData()->bookInfo()->setCoverpage(cover);
                         entries.removeAll(url);
-                        Q_FOREACH(QString entry, entries) {
+                        for(const QString& entry : qAsConst(entries)) {
                             AdvancedComicBookFormat::Page* page = new AdvancedComicBookFormat::Page(acbfDocument);
                             page->setImageHref(entry);
                             acbfDocument->body()->addPage(page);
@@ -1293,3 +1421,90 @@ bool ArchiveBookModel::loadCoMet(QStringList xmlDocuments, QObject *acbfData, QS
     }
     return false;
 }
+
+QString ArchiveBookModel::previewForId(const QString& id) const
+{
+    static const QString directorySplit{"/"};
+    static const QString period{"."};
+    static const QString acbfSuffix{"acbf"};
+    if (d->archive) {
+        if (id.splitRef(directorySplit).last().contains(period)) {
+            const QString suffix = id.splitRef(period).last().toString().toLower();
+            if (d->imageProvider && QImageReader::supportedImageFormats().contains(suffix.toLatin1())) {
+                return QString("image://%1/%2").arg(d->imageProvider->prefix()).arg(id);
+            } else if (suffix == acbfSuffix) {
+                return QString{"image://icon/data-information"};
+            } else {
+                QList<QMimeType> mimetypes = d->mimeDatabase.mimeTypesForFileName(id);
+                if (mimetypes.count() > 0) {
+                    return QString("image://icon/").append(mimetypes.first().iconName());
+                }
+            }
+        } else {
+            return QString{"image://icon/folder"};
+        }
+    }
+    return QString();
+}
+
+QString ArchiveBookModel::fontFamilyName(const QString& fontFileName)
+{
+    QString familyName;
+    if (!fontFileName.isEmpty()) {
+        if (d->fontIdByFilename.contains(fontFileName)) {
+            familyName = QFontDatabase::applicationFontFamilies(d->fontIdByFilename.value(fontFileName)).first();
+        } else {
+            AdvancedComicBookFormat::Document* acbf = qobject_cast<AdvancedComicBookFormat::Document*>(acbfData());
+            if (acbf) {
+                AdvancedComicBookFormat::Binary* binary = qobject_cast<AdvancedComicBookFormat::Binary*>(acbf->objectByID(fontFileName));
+                if (binary) {
+                    int id = QFontDatabase::addApplicationFontFromData(binary->data());
+                    if (id > -1) {
+                        d->fontIdByFilename[fontFileName] = id;
+                        familyName = QFontDatabase::applicationFontFamilies(d->fontIdByFilename.value(fontFileName)).first();
+                    }
+                }
+            }
+            if (familyName.isEmpty()) {
+                QString foundEntry;
+                // If there's more files by the same name, just assume it's the first one in a DFS, because it won't be sensibly deducible anyway
+                for (const QString& entry : d->fileEntries) {
+                    if (entry.endsWith(fontFileName)) {
+                        foundEntry = entry;
+                        break;
+                    }
+                }
+                auto file = archiveFile(foundEntry);
+                if (file) {
+                    int id = QFontDatabase::addApplicationFontFromData(file->data());
+                    if (id > -1) {
+                        d->fontIdByFilename[fontFileName] = id;
+                        familyName = QFontDatabase::applicationFontFamilies(d->fontIdByFilename.value(fontFileName)).first();
+                    }
+                }
+            }
+        }
+    }
+    return familyName;
+}
+
+QString ArchiveBookModel::firstAvailableFont(const QStringList& fontList)
+{
+    QString availableFont;
+    for (const QString& fontName : fontList) {
+        QString actualName = fontName;
+        if (fontName.toLower().endsWith("ttf") || fontName.toLower().endsWith("ttc")) {
+            actualName = fontFamilyName(fontName);
+        }
+        if (actualName.isEmpty()) {
+            if (d->fontDatabase.hasFamily(fontName)) {
+                actualName = fontName;
+            }
+        }
+        if (!actualName.isEmpty()) {
+            availableFont = actualName;
+            break;
+        }
+    }
+    return availableFont;
+}
