@@ -29,10 +29,11 @@
 #include <QIcon>
 #include <QImageReader>
 #include <QPainter>
+#include <QThreadPool>
 
-#include <AcbfDocument.h>
 #include <AcbfBinary.h>
 #include <AcbfData.h>
+#include <AcbfDocument.h>
 
 #include <qtquick_debug.h>
 
@@ -40,9 +41,95 @@ class ArchiveImageProvider::Private
 {
 public:
     Private()
-        : bookModel(nullptr)
-    {}
-    ArchiveBookModel* bookModel;
+    {
+    }
+
+    ArchiveBookModel *bookModel{nullptr};
+    QString prefix;
+};
+
+ArchiveImageProvider::ArchiveImageProvider()
+    : QQuickAsyncImageProvider()
+    , d(new Private)
+{
+}
+
+ArchiveImageProvider::~ArchiveImageProvider()
+{
+    delete d;
+}
+
+class ArchiveImageResponse : public QQuickImageResponse
+{
+public:
+    ArchiveImageResponse(const QString &id, const QSize &requestedSize, ArchiveBookModel *bookModel, const QString &prefix)
+    {
+        m_runnable = new ArchiveImageRunnable(id, requestedSize, bookModel, prefix);
+        m_runnable->setAutoDelete(false);
+        connect(m_runnable, &ArchiveImageRunnable::done, this, &ArchiveImageResponse::handleDone, Qt::QueuedConnection);
+        connect(this, &QQuickImageResponse::finished, m_runnable, &QObject::deleteLater, Qt::QueuedConnection);
+        QThreadPool::globalInstance()->start(m_runnable);
+    }
+
+    void handleDone(QImage image)
+    {
+        m_image = image;
+        Q_EMIT finished();
+    }
+
+    QQuickTextureFactory *textureFactory() const override
+    {
+        return QQuickTextureFactory::textureFactoryForImage(m_image);
+    }
+
+    void cancel() override
+    {
+        m_runnable->abort();
+    }
+
+    ArchiveImageRunnable *m_runnable{nullptr};
+    QImage m_image;
+};
+
+QQuickImageResponse *ArchiveImageProvider::requestImageResponse(const QString &id, const QSize &requestedSize)
+{
+    ArchiveImageResponse *response = new ArchiveImageResponse(id, requestedSize, d->bookModel, d->prefix);
+    return response;
+}
+
+void ArchiveImageProvider::setArchiveBookModel(ArchiveBookModel *model)
+{
+    d->bookModel = model;
+}
+
+void ArchiveImageProvider::setPrefix(QString prefix)
+{
+    d->prefix = prefix;
+}
+
+QString ArchiveImageProvider::prefix() const
+{
+    return d->prefix;
+}
+
+class ArchiveImageRunnable::Private
+{
+public:
+    Private()
+    {
+    }
+    QString id;
+    QSize requestedSize;
+
+    bool abort{false};
+    QMutex abortMutex;
+    bool isAborted()
+    {
+        QMutexLocker locker(&abortMutex);
+        return abort;
+    }
+
+    ArchiveBookModel *bookModel{nullptr};
     QString prefix;
 
     QString errorString;
@@ -62,21 +149,29 @@ public:
     }
 };
 
-ArchiveImageProvider::ArchiveImageProvider()
-    : QQuickImageProvider(QQuickImageProvider::Image)
-    , d(new Private)
+ArchiveImageRunnable::ArchiveImageRunnable(const QString &id, const QSize &requestedSize, ArchiveBookModel *bookModel, const QString &prefix)
+    : d(new Private)
 {
+    d->id = id;
+    d->requestedSize = requestedSize;
+    d->bookModel = bookModel;
+    d->prefix = prefix;
 }
 
-ArchiveImageProvider::~ArchiveImageProvider()
+ArchiveImageRunnable::~ArchiveImageRunnable()
 {
+    abort();
     delete d;
 }
 
-QImage ArchiveImageProvider::requestImage(const QString& id, QSize* size, const QSize& requestedSize)
+void ArchiveImageRunnable::abort()
+{
+    QMutexLocker locker(&d->abortMutex);
+    d->abort = true;
+}
+
+void ArchiveImageRunnable::run() // const QString& id, QSize* size, const QSize& requestedSize)
 {
-    Q_UNUSED(size)
-    Q_UNUSED(requestedSize)
     QImage img;
     bool success = false;
 
@@ -86,48 +181,34 @@ QImage ArchiveImageProvider::requestImage(const QString& id, QSize* size, const
      * see: http://acbf.wikia.com/wiki/Body_Section_Definition#Image
      * TODO: binary files can also handle fonts, and those cannot be loaded into a QImage.
      */
-    if (id.startsWith('#')) {
-        auto document = qobject_cast<AdvancedComicBookFormat::Document*>(d->bookModel->acbfData());
+    if (d->id.startsWith('#')) {
+        auto document = qobject_cast<AdvancedComicBookFormat::Document *>(d->bookModel->acbfData());
 
         if (document) {
-            AdvancedComicBookFormat::Binary* binary = document->data()->binary(id.mid(1));
+            AdvancedComicBookFormat::Binary *binary = qobject_cast<AdvancedComicBookFormat::Binary *>(document->objectByID(d->id.mid(1)));
 
-            if (binary) {
+            if (!d->isAborted() && binary) {
                 success = d->loadImage(&img, binary->data());
             }
         }
     }
 
-    if (!success) {
-        const KArchiveFile* entry = d->bookModel->archiveFile(id);
+    if (!d->isAborted() && !success) {
+        QMutexLocker locker(&d->bookModel->archiveMutex);
+        const KArchiveFile *entry = d->bookModel->archiveFile(d->id);
 
-        if(entry) {
+        if (!d->isAborted() && entry) {
             success = d->loadImage(&img, entry->data());
         }
     }
 
-    if (!success) {
+    if (!d->isAborted() && !success) {
         QIcon oops = QIcon::fromTheme("unknown");
         img = oops.pixmap(oops.availableSizes().last()).toImage();
         QPainter thing(&img);
         thing.drawText(img.rect(), Qt::AlignCenter | Qt::TextWordWrap, d->errorString);
-        qCDebug(QTQUICK_LOG) << "Failed to load image with id:" << id << "and the error" << d->errorString;
+        qCDebug(QTQUICK_LOG) << "Failed to load image with id:" << d->id << "and the error" << d->errorString;
     }
 
-    return img;
-}
-
-void ArchiveImageProvider::setArchiveBookModel(ArchiveBookModel* model)
-{
-    d->bookModel = model;
-}
-
-void ArchiveImageProvider::setPrefix(QString prefix)
-{
-    d->prefix = prefix;
-}
-
-QString ArchiveImageProvider::prefix() const
-{
-    return d->prefix;
+    Q_EMIT done(img);
 }
