@@ -20,6 +20,7 @@
  */
 
 #include "BookModel.h"
+#include "qtquick_debug.h"
 
 #include <AcbfDocument.h>
 
@@ -34,7 +35,7 @@ struct BookPage {
 class BookModel::Private {
 public:
     Private()
-        : currentPage(-1)
+        : currentPage(0)
         , acbfData(nullptr)
         , processing(false)
     {}
@@ -46,6 +47,7 @@ public:
     int currentPage;
     AdvancedComicBookFormat::Document* acbfData;
     bool processing;
+    QString processingDescription;
 };
 
 BookModel::BookModel(QObject* parent)
@@ -108,6 +110,15 @@ void BookModel::addPage(QString url, QString title)
     endInsertRows();
 }
 
+void BookModel::removePage(int pageNumber)
+{
+    QModelIndex index  = createIndex(pageNumber, 0);
+    beginRemoveRows(QModelIndex(), index.row(), index.row());
+    d->entries.removeAt(pageNumber);
+    emit pageCountChanged();
+    endRemoveRows();
+}
+
 void BookModel::clearPages()
 {
     beginResetModel();
@@ -207,12 +218,24 @@ void BookModel::setProcessing(bool processing)
     emit processingChanged();
 }
 
+QString BookModel::processingDescription() const
+{
+    return d->processingDescription;
+}
+
+void BookModel::setProcessingDescription ( const QString& description )
+{
+    d->processingDescription = description;
+    qCDebug(QTQUICK_LOG) << description;
+    Q_EMIT processingDescriptionChanged();
+}
+
 void BookModel::swapPages(int swapThisIndex, int withThisIndex)
 {
     if(swapThisIndex > -1 && withThisIndex > -1 && swapThisIndex < d->entries.count() && withThisIndex < d->entries.count()) {
         QModelIndex firstIndex = createIndex(swapThisIndex, 0);
         QModelIndex secondIndex = createIndex(withThisIndex, 0);
-        d->entries.swap(swapThisIndex, withThisIndex);
+        d->entries.swapItemsAt(swapThisIndex, withThisIndex);
         dataChanged(firstIndex, secondIndex);
     }
 }
