@@ -33,6 +33,8 @@ public:
     ~Private()
     {
         // No deleting the entries - this is done by the master BookListModel already, so do that at your own risk
+        // If we have manually unwrapped any books, though, those need removing
+        qDeleteAll(unwrappedBooks);
     }
     CategoryEntriesModel* q;
     QString name;
@@ -61,14 +63,39 @@ public:
         obj->setProperty("rating", QString::number(entry->rating));
         return obj;
     }
+
+    QList<BookEntry*> unwrappedBooks;
+    BookEntry* unwrapBookEntry(const QObject* obj) {
+        BookEntry* entry = new BookEntry;
+        entry->author = obj->property("author").toStringList();
+        entry->currentPage = obj->property("currentPage").toInt();
+        entry->filename = obj->property("filename").toString();
+        entry->filetitle = obj->property("filetitle").toString();
+        entry->genres = obj->property("genres").toStringList();
+        entry->keywords = obj->property("keywords").toStringList();
+        entry->characters = obj->property("characters").toStringList();
+        entry->created = obj->property("created").toDateTime();
+        entry->lastOpenedTime = obj->property("lastOpenedTime").toDateTime();
+        entry->publisher = obj->property("publisher").toString();
+        entry->series = obj->property("series").toStringList();
+        entry->title = obj->property("title").toString();
+        entry->totalPages = obj->property("totalPages").toInt();
+        entry->thumbnail = obj->property("thumbnail").toString();
+        entry->description = obj->property("description").toStringList();
+        entry->comment = obj->property("comment").toString();
+        entry->tags = obj->property("tags").toStringList();
+        entry->rating = obj->property("rating").toInt();
+        unwrappedBooks << entry;
+        return entry;
+    }
 };
 
 CategoryEntriesModel::CategoryEntriesModel(QObject* parent)
     : QAbstractListModel(parent)
     , d(new Private(this))
 {
-    connect(this, SIGNAL(entryDataUpdated(BookEntry*)), this, SLOT(entryDataChanged(BookEntry*)));
-    connect(this, SIGNAL(entryRemoved(BookEntry*)), this, SLOT(entryRemove(BookEntry*)));
+    connect(this, &CategoryEntriesModel::entryDataUpdated, this, &CategoryEntriesModel::entryDataChanged);
+    connect(this, &CategoryEntriesModel::entryRemoved, this, &CategoryEntriesModel::entryRemove);
 }
 
 CategoryEntriesModel::~CategoryEntriesModel()
@@ -216,59 +243,89 @@ int CategoryEntriesModel::rowCount(const QModelIndex& parent) const
     return d->categoryModels.count() + d->entries.count();
 }
 
+int CategoryEntriesModel::count() const
+{
+    return rowCount();
+}
+
 void CategoryEntriesModel::append(BookEntry* entry, Roles compareRole)
 {
     int insertionIndex = 0;
-    int seriesOne = -1; int seriesTwo = -1;
-    if(compareRole == SeriesRole) {
-        seriesOne = entry->series.indexOf(name());
-        if (entry->series.contains(name(), Qt::CaseInsensitive) && seriesOne == -1){
-            for (int s=0; s<entry->series.size();s++) {
-                if (name().toLower() == entry->series.at(s).toLower()) {
-                    seriesOne = s;
-                }
-            }
-        }
+    if(compareRole == UnknownRole) {
+        // If we don't know what order to sort by, literally just append the entry
+        insertionIndex = d->entries.count();
     }
-    for(; insertionIndex < d->entries.count(); ++insertionIndex)
-    {
+    else {
+        int seriesOne = -1; int seriesTwo = -1;
         if(compareRole == SeriesRole) {
-            seriesTwo = d->entries.at(insertionIndex)->series.indexOf(name());
-            if ( d->entries.at(insertionIndex)->series.contains(name(), Qt::CaseInsensitive) && seriesTwo == -1){
-                for (int s=0; s< d->entries.at(insertionIndex)->series.size();s++) {
-                    if (name().toLower() ==  d->entries.at(insertionIndex)->series.at(s).toLower()) {
-                        seriesTwo = s;
+            seriesOne = entry->series.indexOf(name());
+            if (entry->series.contains(name(), Qt::CaseInsensitive) && seriesOne == -1){
+                for (int s=0; s<entry->series.size();s++) {
+                    if (QString::compare(name(), entry->series.at(s), Qt::CaseInsensitive)) {
+                        seriesOne = s;
                     }
                 }
             }
         }
-        if(compareRole == CreatedRole)
+        for(; insertionIndex < d->entries.count(); ++insertionIndex)
         {
-            if(entry->created <= d->entries.at(insertionIndex)->created)
-            { continue; }
-            break;
-        }
-        else if((seriesOne>-1 && seriesTwo>-1)
-                && entry->seriesNumbers.at(seriesOne).toInt() > 0
-                && d->entries.at(insertionIndex)->seriesNumbers.at(seriesTwo).toInt() > 0)
-        {
-            if (entry->seriesVolumes.at(seriesOne).toInt() >= d->entries.at(insertionIndex)->seriesVolumes.at(seriesTwo).toInt()
-                    && entry->seriesNumbers.at(seriesOne).toInt() > d->entries.at(insertionIndex)->seriesNumbers.at(seriesTwo).toInt())
-            {continue;}
-            break;
-        }
-        else
-        {
-            if(QString::localeAwareCompare(d->entries.at(insertionIndex)->title, entry->title) > 0)
-            { break; }
+            if(compareRole == SeriesRole) {
+                seriesTwo = d->entries.at(insertionIndex)->series.indexOf(name());
+                if ( d->entries.at(insertionIndex)->series.contains(name(), Qt::CaseInsensitive) && seriesTwo == -1){
+                    for (int s=0; s< d->entries.at(insertionIndex)->series.size();s++) {
+                        if (QString::compare(name(), d->entries.at(insertionIndex)->series.at(s), Qt::CaseInsensitive)) {
+                            seriesTwo = s;
+                        }
+                    }
+                }
+            }
+            if(compareRole == CreatedRole)
+            {
+                if(entry->created <= d->entries.at(insertionIndex)->created)
+                { continue; }
+                break;
+            }
+            else if((seriesOne>-1 && seriesTwo>-1)
+                    && entry->seriesNumbers.count() > -1 && entry->seriesNumbers.count() > seriesOne
+                    && d->entries.at(insertionIndex)->seriesNumbers.count() > -1 && d->entries.at(insertionIndex)->seriesNumbers.count() > seriesTwo
+                    && entry->seriesNumbers.at(seriesOne).toInt() > 0
+                    && d->entries.at(insertionIndex)->seriesNumbers.at(seriesTwo).toInt() > 0)
+            {
+                if (entry->seriesVolumes.count() > -1 && entry->seriesVolumes.count() > seriesOne
+                        && d->entries.at(insertionIndex)->seriesVolumes.count() > -1 && d->entries.at(insertionIndex)->seriesVolumes.count() > seriesTwo
+                        && entry->seriesVolumes.at(seriesOne).toInt() >= d->entries.at(insertionIndex)->seriesVolumes.at(seriesTwo).toInt()
+                        && entry->seriesNumbers.at(seriesOne).toInt() > d->entries.at(insertionIndex)->seriesNumbers.at(seriesTwo).toInt())
+                {continue;}
+                break;
+            }
+            else
+            {
+                if(QString::localeAwareCompare(d->entries.at(insertionIndex)->title, entry->title) > 0)
+                { break; }
+            }
         }
     }
     beginInsertRows(QModelIndex(), insertionIndex, insertionIndex);
     d->entries.insert(insertionIndex, entry);
+    Q_EMIT countChanged();
     endInsertRows();
 }
 
-QString CategoryEntriesModel::name() const
+void CategoryEntriesModel::appendFakeBook ( QObject* book, CategoryEntriesModel::Roles compareRole )
+{
+    append(d->unwrapBookEntry(book), compareRole);
+}
+
+void CategoryEntriesModel::clear()
+{
+    beginResetModel();
+    qDeleteAll(d->unwrappedBooks);
+    d->unwrappedBooks.clear();
+    d->entries.clear();
+    endResetModel();
+}
+
+const QString &CategoryEntriesModel::name() const
 {
     return d->name;
 }
@@ -289,7 +346,7 @@ QObject * CategoryEntriesModel::leafModelForEntry(BookEntry* entry)
     }
     else
     {
-        Q_FOREACH(CategoryEntriesModel* testModel, d->categoryModels)
+        for(CategoryEntriesModel* testModel : d->categoryModels)
         {
             model = testModel->leafModelForEntry(entry);
             if(model) {
@@ -304,12 +361,16 @@ void CategoryEntriesModel::addCategoryEntry(const QString& categoryName, BookEnt
 {
     if(categoryName.length() > 0)
     {
-        QStringList splitName = categoryName.split("/");
-        QString nextCategory = splitName.takeFirst();
+        static const QString splitString{"/"};
+        int splitPos = categoryName.indexOf(splitString);
+        QString desiredCategory{categoryName};
+        if(splitPos > -1) {
+            desiredCategory = categoryName.left(splitPos);
+        }
         CategoryEntriesModel* categoryModel = nullptr;
-        Q_FOREACH(CategoryEntriesModel* existingModel, d->categoryModels)
+        for(CategoryEntriesModel *existingModel : qAsConst(d->categoryModels))
         {
-            if(existingModel->name().toLower() == nextCategory.toLower())
+            if(QString::compare(existingModel->name(), desiredCategory, Qt::CaseInsensitive) == 0)
             {
                 categoryModel = existingModel;
                 break;
@@ -318,9 +379,9 @@ void CategoryEntriesModel::addCategoryEntry(const QString& categoryName, BookEnt
         if(!categoryModel)
         {
             categoryModel = new CategoryEntriesModel(this);
-            connect(this, SIGNAL(entryDataUpdated(BookEntry*)), categoryModel, SIGNAL(entryDataUpdated(BookEntry*)));
-            connect(this, SIGNAL(entryRemoved(BookEntry*)), categoryModel, SIGNAL(entryRemoved(BookEntry*)));
-            categoryModel->setName(nextCategory);
+            connect(this, &CategoryEntriesModel::entryDataUpdated, categoryModel, &CategoryEntriesModel::entryDataUpdated);
+            connect(this, &CategoryEntriesModel::entryRemoved, categoryModel, &CategoryEntriesModel::entryRemoved);
+            categoryModel->setName(desiredCategory);
 
             int insertionIndex = 0;
             for(; insertionIndex < d->categoryModels.count(); ++insertionIndex)
@@ -337,7 +398,8 @@ void CategoryEntriesModel::addCategoryEntry(const QString& categoryName, BookEnt
         if (categoryModel->indexOfFile(entry->filename) == -1) {
             categoryModel->append(entry, compareRole);
         }
-        categoryModel->addCategoryEntry(splitName.join("/"), entry);
+        if(splitPos > -1)
+            categoryModel->addCategoryEntry(categoryName.mid(splitPos + 1), entry);
     }
 }
 
@@ -358,12 +420,22 @@ QObject* CategoryEntriesModel::get(int index)
     return obj;
 }
 
-int CategoryEntriesModel::indexOfFile(QString filename)
+BookEntry * CategoryEntriesModel::getBookEntry ( int index )
+{
+    BookEntry* entry{nullptr};
+    if(index > -1 && index < d->entries.count())
+    {
+        entry = d->entries.at(index);
+    }
+    return entry;
+}
+
+int CategoryEntriesModel::indexOfFile(const QString& filename)
 {
     int index = -1, i = 0;
     if(QFile::exists(filename))
     {
-        Q_FOREACH(BookEntry* entry, d->entries)
+        for(BookEntry* entry : d->entries)
         {
             if(entry->filename == filename)
             {
@@ -392,14 +464,11 @@ int CategoryEntriesModel::bookCount() const
 QObject* CategoryEntriesModel::getEntry(int index)
 {
     PropertyContainer* obj = new PropertyContainer("book", this);
-    if(index < 0 && index > rowCount() -1) {
-        // don't be a silly person, you can't get a nothing...
-    }
-    else if(index > d->categoryModels.count() - 1) {
+    if(index > d->categoryModels.count() - 1 && index < rowCount()) {
         // This is a book - get a book!
         obj = qobject_cast<PropertyContainer*>(get(index - d->categoryModels.count()));
     }
-    else {
+    else if (index >= 0 && index < d->categoryModels.count()) {
         CategoryEntriesModel* catEntry = d->categoryModels.at(index);
         obj->setProperty("title", catEntry->name());
         obj->setProperty("categoryEntriesCount", catEntry->bookCount());
@@ -408,14 +477,14 @@ QObject* CategoryEntriesModel::getEntry(int index)
     return obj;
 }
 
-QObject* CategoryEntriesModel::bookFromFile(QString filename)
+QObject* CategoryEntriesModel::bookFromFile(const QString& filename)
 {
     PropertyContainer* obj = qobject_cast<PropertyContainer*>(get(indexOfFile(filename)));
     if(obj->property("filename").toString().isEmpty()) {
         if(QFileInfo::exists(filename)) {
             QFileInfo info(filename);
             obj->setProperty("title", info.completeBaseName());
-            obj->setProperty("created", info.created());
+            obj->setProperty("created", info.birthTime());
 
             KFileMetaData::UserMetaData data(filename);
             if (data.hasAttribute("peruse.currentPage")) {
@@ -436,7 +505,7 @@ QObject* CategoryEntriesModel::bookFromFile(QString filename)
             obj->setProperty("filename", filename);
 
             QString thumbnail;
-            if(filename.toLower().endsWith("cbr")) {
+            if(filename.toLower().endsWith("cbr") || filename.toLower().endsWith("cbz")) {
                 thumbnail = QString("image://comiccover/").append(filename);
             }
 #ifdef USE_PERUSE_PDFTHUMBNAILER
