@@ -23,15 +23,26 @@
 #define FOLDERBOOKMODEL_H
 
 #include "BookModel.h"
-
+#include <qqmlintegration.h>
+/**
+ * \brief Class to handle a folder of images as a book.
+ *
+ * This class will treat a folder of images as a book, with the alphabetical
+ * order of the files determining the page order.
+ */
 class FolderBookModel : public BookModel
 {
     Q_OBJECT
+    QML_ELEMENT
 public:
-    explicit FolderBookModel(QObject* parent = 0);
-    virtual ~FolderBookModel();
+    explicit FolderBookModel(QObject *parent = nullptr);
+    ~FolderBookModel() override;
 
-    virtual void setFilename(QString newFilename);
+    /**
+     * \brief Set the folder name of this book or a file describing
+     * describing the images that belong to this book.
+     */
+    void setFilename(QString newFilename) override;
 };
 
-#endif//FOLDERBOOKMODEL_H
+#endif // FOLDERBOOKMODEL_H
