@@ -21,33 +21,38 @@
 
 #include "PeruseConfig.h"
 
-#include <KFileMetaData/UserMetaData>
 #include <KConfig>
 #include <KConfigGroup>
+#include <KFileMetaData/UserMetaData>
+#include <KNSCore/EngineBase>
 
-#include <QTimer>
 #include <QFile>
+#include <QFileInfo>
+#include <QImageReader>
+#include <QMimeDatabase>
+#include <QTimer>
+
+using namespace Qt::Literals::StringLiterals;
 
 class PeruseConfig::Private
 {
 public:
     Private()
-        : config("peruserc")
-    {};
+        : config("peruserc") { };
     KConfig config;
 };
 
-PeruseConfig::PeruseConfig(QObject* parent)
+PeruseConfig::PeruseConfig(QObject *parent)
     : QObject(parent)
     , d(new Private)
 {
-    QStringList locations = d->config.group("general").readEntry("book locations", QStringList());
-    if(locations.count() < 1)
-    {
+    QStringList locations = d->config.group(u"general"_s).readEntry("book locations", QStringList());
+    if (locations.count() < 1) {
         locations = QStandardPaths::standardLocations(QStandardPaths::DocumentsLocation);
         locations << QStandardPaths::standardLocations(QStandardPaths::DownloadLocation);
         locations << QString("%1/comics").arg(QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation).first());
-        d->config.group("general").writeEntry("book locations", locations);
+        d->config.group(u"general"_s).writeEntry(u"book locations"_s, locations);
+        d->config.group(u"general"_s).writeEntry(u"animate jump areas"_s, true);
         d->config.sync();
     }
 }
@@ -62,94 +67,112 @@ void PeruseConfig::bookOpened(QString path)
     QStringList recent = recentlyOpened();
 
     int i = recent.indexOf(path);
-    if(i == 0)
-    {
+    if (i == 0) {
         // This is already first, don't do work we don't need to, because that's just silly
         return;
-    }
-    else
-    {
+    } else {
         recent.removeAll(path);
         recent.prepend(path);
     }
-    d->config.group("general").writeEntry("recently opened", recent);
+    d->config.group(u"general"_s).writeEntry(u"recently opened"_s, recent);
     d->config.sync();
     emit recentlyOpenedChanged();
 }
 
 QStringList PeruseConfig::recentlyOpened() const
 {
-    QStringList recent = d->config.group("general").readEntry("recently opened", QStringList());
+    QStringList recent = d->config.group(u"general"_s).readEntry(u"recently opened"_s, QStringList());
     QStringList actualRecent;
-    while(recent.count() > 0) {
+    while (recent.count() > 0) {
         QString current = recent.takeFirst();
-        if(QFile::exists(current)) {
+        if (QFile::exists(current)) {
             actualRecent.append(current);
         }
     }
     return actualRecent;
 }
 
-void PeruseConfig::addBookLocation(const QString& location)
+void PeruseConfig::addBookLocation(const QString &location)
 {
-    if(location.startsWith("file://"))
-    {
+    if (location.startsWith("file://")) {
 #ifdef Q_OS_WIN
         QString newLocation = location.mid(8);
 #else
         QString newLocation = location.mid(7);
 #endif
-        QStringList locations = bookLocations();
+        const QStringList locations = bookLocations();
         // First, get rid of all the entries which start with the newly added location, because that's silly
         QStringList newLocations;
         bool alreadyInThere = false;
-        Q_FOREACH(QString entry, locations) {
-            if(!entry.startsWith(newLocation))
-            {
+        for (const QString &entry : locations) {
+            if (!entry.startsWith(newLocation)) {
                 newLocations.append(entry);
             }
-            if(newLocation.startsWith(entry))
-            {
+            if (newLocation.startsWith(entry)) {
                 alreadyInThere = true;
             }
         }
-        if(alreadyInThere)
-        {
+        if (alreadyInThere) {
             // Don't be silly, don't add a new location if it's already covered by something more high level...
             emit showMessage("Attempted to add a new location to the list of search folders which is a sub-folder to something already in the list.");
             return;
         }
         newLocations.append(newLocation);
-        d->config.group("general").writeEntry("book locations", newLocations);
+        d->config.group(u"general"_s).writeEntry(u"book locations"_s, newLocations);
         d->config.sync();
         emit bookLocationsChanged();
     }
 }
 
-void PeruseConfig::removeBookLocation(const QString& location)
+void PeruseConfig::removeBookLocation(const QString &location)
 {
     QStringList locations = bookLocations();
     locations.removeAll(location);
-    d->config.group("general").writeEntry("book locations", locations);
+    d->config.group(u"general"_s).writeEntry(u"book locations"_s, locations);
     d->config.sync();
     QTimer::singleShot(100, this, SIGNAL(bookLocationsChanged()));
 }
 
 QStringList PeruseConfig::bookLocations() const
 {
-    QStringList locations = d->config.group("general").readEntry("book locations", QStringList());
+    QStringList locations = d->config.group(u"general"_s).readEntry(u"book locations"_s, QStringList());
     return locations;
 }
 
 QString PeruseConfig::newstuffLocation() const
 {
-    QString knsrc = QStandardPaths::locate(QStandardPaths::GenericConfigLocation, "peruse.knsrc");
-    if(qEnvironmentVariableIsSet("APPDIR"))
-    {
+    const QStringList configFiles = KNSCore::EngineBase::availableConfigFiles().filter(QRegularExpression(u"pereuse.knsrc$"_s));
+    if (configFiles.isEmpty()) {
+        return {};
+    }
+
+    if (!qEnvironmentVariableIsSet("APPDIR")) {
+        return configFiles[0];
+    }
+
+    for (const auto &configFile : configFiles) {
         // Because appimage install happens into /app/usr...
+        auto knsrc = configFiles[0];
         knsrc = knsrc.prepend("/usr").prepend(qgetenv("APPDIR"));
+        return knsrc;
+    }
+    return {};
+}
+
+bool PeruseConfig::animateJumpAreas() const
+{
+    return d->config.group(u"general"_s).readEntry(u"animate jump areas"_s, true);
+}
+
+void PeruseConfig::setAnimateJumpAreas(bool animate)
+{
+    bool animateJumpAreasCurrentVal = animateJumpAreas();
+
+    if (animateJumpAreasCurrentVal != animate) {
+        d->config.group(u"general"_s).writeEntry(u"animate jump areas"_s, animate);
+        d->config.sync();
+        emit animateJumpAreasChanged();
     }
-    return knsrc;
 }
 
 QString PeruseConfig::homeDir() const
@@ -181,8 +204,23 @@ QString PeruseConfig::getFilesystemProperty(QString fileName, QString propertyNa
         value = data.tags().join(",");
     } else if (propertyName == "comment") {
         value = data.userComment();
+    } else if (propertyName == "bytes") {
+        value = QString::number(QFileInfo(fileName).size());
+    } else if (propertyName == "mimetype") {
+        QMimeDatabase db;
+        QMimeType mime = db.mimeTypeForFile(fileName);
+        value = mime.name();
     } else {
         value = data.attribute(QString("peruse.").append(propertyName));
     }
     return value;
 }
+
+QStringList PeruseConfig::supportedImageFormats() const
+{
+    QStringList formats;
+    for (const QByteArray &format : QImageReader::supportedImageFormats()) {
+        formats << format;
+    }
+    return formats;
+}
