@@ -23,14 +23,20 @@
 #define _QML_PLUGIN_H
 
 #include <QQmlExtensionPlugin>
-
+/**
+ * \brief The QML plugin file.
+ *
+ * This takes the different classes in this folder and turns them into
+ * a QML plugin, so the properties can be accessed and modified from a
+ * QML file.
+ */
 class QmlPlugins : public QQmlExtensionPlugin
 {
     Q_OBJECT
     Q_PLUGIN_METADATA(IID "org.qt-project.Qt.QQmlExtensionInterface")
 public:
-    virtual void initializeEngine(QQmlEngine *engine, const char *uri);
-    virtual void registerTypes(const char *uri);
+    void initializeEngine(QQmlEngine *engine, const char *uri) override;
+    void registerTypes(const char *uri) override;
 };
 
 #endif
